@@ -1,16 +1,3 @@
-#' @rawNamespace if (packageVersion("parallelly") <= "1.43.0") {
-#'   S3method(all.equal,connection)
-#' }
-if (packageVersion("parallelly") <= "1.43.0") {
-  all.equal.connection <- function(target, current, ...) {
-    if (!identical(target, current)) {
-      return("Connections differ")
-    }
-    TRUE
-  }
-}
-
-
 ## This is needed in order to be able to assert that we later
 ## actually work with the same connection.  See R-devel thread
 ## 'closeAllConnections() can really mess things up' on 2016-10-30
